function log(l) {
	if (l >= 0) {
		console.log(arguments)
	}
}



function graph_01(svgele, data_url, graph) {
	let svg = d3.select(svgele)
	let margin = {top: 20, right: 20, bottom: 50, left: 100}

	x = ["width", "height"].map(function(x){return parseInt(svg.style(x),10)})

	let width = x[0] - margin.left - margin.right
	let height = x[1] - margin.top - margin.bottom

	let g = svg.append("g").attr("transform", "translate(" + margin.left + "," + margin.top + ")")

	//var parseTime = d3.timeParse("%d-%b-%y");

	//var x = d3.scaleTime().rangeRound([0, width]);
	//let scx = d3.scaleLinear().range([0, width])
	//let scx = d3.scaleLog().range([0, width])
	let scx = graph.log_x ? d3.scaleLog().range([0, width])  :  d3.scaleLinear().range([0, width])
	let scy = graph.log_y ? d3.scaleLog().range([height, 0])  :  d3.scaleLinear().range([height, 0])

	//let fx = function(d){return Math.log(d.m_s)/Math.log(2)}
	let fx = graph.fx
	let fy = graph.fy
	let fy_dev = graph.fy_dev

	let cb_render = function(error, res) {
		if (error) throw error;

		let scale_margin = function(a, f) {
			if (f === undefined) f = 0.1
			let b = a.range()
			let d = b[1] - b[0]
			let c = [
				a.invert(b[0] - f * d),
				a.invert(b[1] + f * d)
			]
			a.domain(c)
		}

		let data = res.rows
		scx.domain(d3.extent(data, fx))
		scy.domain(d3.extent(data, fy))
		scale_margin(scx)
		scale_margin(scy, 0.25)

		if (graph.setup_scale_y !== undefined) {
			graph.setup_scale_y(scy)
		}

		let axis_x = d3.axisBottom(scx)
			//.ticks(4)
			.tickSizeOuter(0)
			.tickSizeInner(-height)
			//.ticks(2, ".2f")

		let axis_y = d3.axisLeft(scy)
			//.ticks(4)
			.tickSizeOuter(0)
			.tickSizeInner(-width)

		if (graph.setup_axis_x !== undefined) {
			graph.setup_axis_x(axis_x)
		}
		if (graph.setup_axis_y !== undefined) {
			graph.setup_axis_y(axis_y)
		}

		g.append("g")
			.attr("class", "axis axis--x")
			.attr("transform", "translate(0," + (height) + ")")
			.call(axis_x)
			.append("text")
			.attr("fill", "#000")
			.attr("x", width)
			.attr("dy", "2.5em")
			.style("text-anchor", "end")
			.text(graph.xlabel)

		g.append("g")
			.attr("class", "axis axis--y")
			.call(axis_y)
			.append("text")
			.attr("fill", "#000")
			.attr("transform", "rotate(-90)")
			.attr("y", 0)
			.attr("dx", "0.0em")
			.attr("dy", "-3.5em")
			.style("text-anchor", "end")
			.text(graph.ylabel)

		let a = graph.sets
		for (let i1 = 0; i1 < a.length; ++i1) {
			render_set(a[i1], res, i1)
		}
	}

	let render_set = function(set, res, setix) {
		let data = res.rows.slice(0)
		if (set.filter_datum !== undefined) {
			let a = data
			for (let i1 = 0; i1 < a.length; ++i1) {
				if (!set.filter_datum(a[i1])) {
					a.splice(i1, 1)
					--i1;
				}
			}
		}

		let line = d3.line()
			//.defined(function(d) { return d; })
			.x(function(d){return scx(fx(d))})
			.y(function(d){return scy(fy(d))})

		g.append("path")
			.datum(data)
			.attr("class", "line line-"+setix)
			.attr("d", line)
			//.attr("stroke-linecap", "round")

		g.selectAll(".dot .dot-"+setix)
			.data(data)
			.enter().append("circle")
				.attr("class", "dot dot-"+setix)
				.attr("cx", line.x())
				.attr("cy", line.y())
				.attr("r", 3.5)


		// make error data
		if (graph.fy_dev !== undefined) {
			let fy_dev = graph.fy_dev
			let a = data
			for (let i1 = 0; i1 < a.length; ++i1) {
				let x = fx(a[i1])
				let line_error = d3.line()
					.x(scx(x))
					.y(function(d){return scy(d)})
				g.append("path")
					.datum([fy(a[i1]) - fy_dev(a[i1]), fy(a[i1]) + fy_dev(a[i1])])
					.attr("class", "line line-"+setix)
					.attr("d", line_error)
			}
		}

	}
	d3.json(data_url, cb_render)
}





function deck_init() {
	let tickv = [2, 4, 8, 16, 32, 64, 128, 256, 512]
	graph_01("#grgr01", "data-02.json", {
		xlabel: "Messages / s",
		ylabel: "Round trip (ms)",
		log_x: true,
		log_y: true,
		fx: function(d){return d.m_s},
		fy: function(d){return d.RTavg},
		fy_dev: function(d){return d.RTdev},
		setup_axis_x: function(x) {x.tickValues(tickv).tickFormat(d3.format("d"))},
		setup_axis_y: function(x) {x.ticks(4, "d")},
		sets: [
			{
				filter_datum: function(d){return d.b_s == 20}
			},
			{
				filter_datum: function(d){return d.b_s == 30}
			},
			{
				filter_datum: function(d){return d.b_s == 40}
			}
		]
	})



	graph_01("#grgr02", "data-02.json", {
		xlabel: "Messages / s",
		ylabel: "CPU broker",
		log_x: true,
		fx: function(d){return d.m_s},
		fy: function(d){return d.cpu_broker},
		fy_dev: function(d){return 0.0},
		setup_scale_y: function(x){x.domain([0,x.domain()[1]])},
		setup_axis_x: function(x) {x.tickValues(tickv).tickFormat(d3.format("d"))},
		setup_axis_y: function(x) {x.ticks(4, ".2f")},
		sets: [
			{
				filter_datum: function(d){return d.b_s == 20}
			},
			{
				filter_datum: function(d){return d.b_s == 30}
			},
			{
				filter_datum: function(d){return d.b_s == 40}
			}
		]
	})



	graph_01("#grgr03", "data-02.json", {
		xlabel: "Messages / s",
		ylabel: "CPU others",
		log_x: true,
		fx: function(d){return d.m_s},
		fy: function(d){return d.cpu_other},
		fy_dev: function(d){return 0.0},
		setup_scale_y: function(x){x.domain([0,x.domain()[1]])},
		setup_axis_x: function(x) {x.tickValues(tickv).tickFormat(d3.format("d"))},
		setup_axis_y: function(x) {x.ticks(4, ".2f")},
		sets: [
			{
				filter_datum: function(d){return d.b_s == 20}
			},
			{
				filter_datum: function(d){return d.b_s == 30}
			},
			{
				filter_datum: function(d){return d.b_s == 40}
			}
		]
	})



	graph_01("#grgr_cpu_broker_linear", "data-02.json", {
		xlabel: "Bandwidth MB/s",
		ylabel: "CPU broker",
		fx: function(d){return d.b_s},
		fy: function(d){return d.cpu_other},
		fy_dev: function(d){return 0.0},
		setup_scale_y: function(x){x.domain([0,x.domain()[1]])},
		setup_axis_x: function(x) {x.tickValues([20, 30, 40]).tickFormat(d3.format("d"))},
		setup_axis_y: function(x) {x.ticks(4, ".2f")},
		sets: [
			{
				filter_datum: function(d){return d.m_s == 64}
			},
			{
				filter_datum: function(d){return d.m_s == 128}
			},
			{
				filter_datum: function(d){return d.m_s == 256}
			},
			{
				filter_datum: function(d){return d.m_s == 512}
			}
		]
	})
}
